/**
 *#########################################################################
 *
 * A component of the GAI application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: Chi-Yu Huang, Greenstone Digital Library, University of Waikato
 * Modified: 03.2005
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.admin.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.List;
import java.util.ArrayList;
import java.lang.Object;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import java.awt.event.MouseEvent;
import java.net.*;
import java.sql.*;

import org.greenstone.admin.GAI;
import org.greenstone.admin.GAIManager;
import org.greenstone.admin.gui.ThreadControl;
import org.greenstone.core.Configuration;
import org.greenstone.core.Dictionary;
import org.greenstone.core.util.Utility;
import org.greenstone.core.ParsingProgress;

/** The Configuration pane is to view the status of relevant configure files in GSIII
 * @author Chi-Yu Huang, Greenstone Digital Library, University of Waikato
 * @version
 */
public class ConfPane
    extends JPanel
    implements ActionListener {
    
    /* The pane to demonstrate Configuration information, including the build property file
     * being monitored and their content, and control functions */
    protected JSplitPane main_conf_pane = null;
    
    protected JPanel main_contentPane = null;
    
    /** The panel that contains a Conf_list */
    private JPanel confList_pane =  null;
    
    /** The panel that contains the conf content. */
    private JPanel confContent_pane =  null;
    private JScrollPane conf_table_pane = null;

    /** The List showing all the Configuration files concerned. */
    private JList conf_list=null;
    private JTree conf_tree = null;

    /** The pane area into which the Configuration content is placed. */
    //private JScrollPane conf_content = null;

    /** The label at the top of the confList_pane. */
    private JLabel confList_label =  null;
    /** The label shown at the top of the confContent Pane. */
    private JLabel confContent_label  =  null;

    // The control buttons used to manipulate Configuration Pane
    protected JPanel outter_button_pane = null;
    protected JPanel outter_control_pane = null;
    protected JPanel inner_button_pane = null;
    protected JPanel inner_control_pane = null;
    protected JPanel tomcat_button_pane = null;
    protected JPanel mysql_button_pane = null;
    protected JPanel setting_shown_pane = null;
    
    /**Buttons in inner button pane */
    private JButton startup_tomcat_button = null;
    private JButton shutdown_tomcat_button = null;
    private JButton restart_tomcat_button = null;
    private JButton startup_mysql_button = null;
    private JButton shutdown_mysql_button = null;
    private JButton restart_mysql_button = null;
      
    private JButton save_button = null;

    /** Buttons in outter button pane*/
    //private JButton reload_button = null;
    //private JButton delete_button = null;
    //private JButton exit_button = null;

    private JLabel conf_label = null;
    private JLabel conf_setting_label = null;
    private JTextField conf_field = null;

    //control tomcat and mysql server running status
    private String tomcat_server;
    private String tomcat_port;
    private String mysql_adminuser;
    private String mysql_server;
    private String mysql_port;
    private boolean tomcat_server_up = false;
    private boolean mysql_server_up = false;

    protected JTextArea text_area = null;
      
    /** The various sizes for the screen layout*/
    static private Dimension MIN_SIZE = new Dimension( 90,  90);
    static private Dimension LIST_SIZE  = new Dimension(200, 450);
    static private Dimension CONTENT_SIZE = new Dimension (600,450);
    static private Dimension TABLE_SIZE = new Dimension(500,200);
    
    private JTable conf_setting_table = null;

    private int num_of_setting;
    private int num_of_change = 0;

    //The details display in the table
    int rowLength = 20;
    int colLength = 3;
    private Object[][] conf_display = new Object [rowLength][colLength];
    private Object[][] conf_setting;
    
    //An array to store all the details in the build.properties file
    private ArrayList conf_array;
    
    // all the relevant details from the build.properties
    private static boolean valueChanged =false;
    
    ConfTableModel conf_table_model;
    DefaultMutableTreeNode top = new DefaultMutableTreeNode("Configuration Files");

    //Constructor
    public ConfPane() {
	// create all the control button panes
	inner_control_pane = new JPanel();
	inner_button_pane = new JPanel();
	tomcat_button_pane = new JPanel();
	mysql_button_pane = new JPanel();
	outter_control_pane = new JPanel();
	outter_button_pane = new JPanel();
	
	// Main Configuration Pane
	main_conf_pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
       
	// Main pane for Configuration Content Pane
	main_contentPane = new JPanel();
	
	//Create a tree for a list Configuration files we are interested
	DefaultMutableTreeNode child1 = new DefaultMutableTreeNode("Build");
	DefaultMutableTreeNode compile = new DefaultMutableTreeNode("Compile");
	DefaultMutableTreeNode tomcat = new DefaultMutableTreeNode("Tomcat");
	DefaultMutableTreeNode proxy = new DefaultMutableTreeNode("Proxy");
	DefaultMutableTreeNode mysql = new DefaultMutableTreeNode("Mysql");	
	DefaultMutableTreeNode gsdl = new DefaultMutableTreeNode("GSDL");
	child1.add(compile);
	child1.add(tomcat);
	child1.add(proxy);
	child1.add(mysql);
	child1.add(gsdl);

	DefaultMutableTreeNode child2 =
	    new DefaultMutableTreeNode("Conf 2");
	DefaultMutableTreeNode child3 =
	    new DefaultMutableTreeNode("Conf 3");
	top.add(child1);
	top.add(child2);
	top.add(child3);
	conf_tree = new JTree(top);
	conf_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	conf_tree.addTreeSelectionListener(new ConfTreeListener());

	// inner button_pane Buttons
	StartupTomcatButtonListener stbl = new StartupTomcatButtonListener();
	ShutdownTomcatButtonListener stdbl = new ShutdownTomcatButtonListener();
	RestartTomcatButtonListener rtbl = new RestartTomcatButtonListener();

	StartupMysqlButtonListener smbl = new StartupMysqlButtonListener();
	ShutdownMysqlButtonListener smdbl = new ShutdownMysqlButtonListener();
	RestartMysqlButtonListener rmbl = new RestartMysqlButtonListener();
	
	//save button
	SaveButtonListener sabl = new SaveButtonListener();
	
	//outter button_pane Buttons
	//ReloadButtonListener rbl = new ReloadButtonListener();
	//DeleteButtonListener dbl = new DeleteButtonListener();
	//ExitButtonListener ebl = new ExitButtonListener();
	//ImageIcon reloadButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/Refresh16.gif");
	//ImageIcon deleteButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/Delete16.gif");
	//ImageIcon exitButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/exit16.gif");

	ImageIcon startupTomcatButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/redo16.gif");
	ImageIcon shutdownTomcatButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/stop16.gif");
	ImageIcon restartTomcatButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/stop16.gif");
	
	ImageIcon startupMysqlButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/redo16.gif");
	ImageIcon shutdownMysqlButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/stop16.gif");
	ImageIcon restartMysqlButtonIcon = new ImageIcon(GAI.admin_path + "images/toolbarButtonGraphics/general/stop16.gif");
	ImageIcon saveButtonIcon = new ImageIcon(GAI.admin_path  + "images/toolbarButtonGraphics/general/Save16.gif");

	startup_tomcat_button = new JButton("Startup Tomcat", startupTomcatButtonIcon);
	startup_tomcat_button.addActionListener(stbl);
	startup_tomcat_button.setMnemonic(KeyEvent.VK_S);
	startup_tomcat_button.setToolTipText("Click this button to startup the Tomcat server");
	startup_tomcat_button.setEnabled(false);

	shutdown_tomcat_button = new JButton("Shutdown Tomcat", shutdownTomcatButtonIcon);
	shutdown_tomcat_button.addActionListener(stdbl);
	shutdown_tomcat_button.setEnabled(false);
	//shutdown_button.setMnemonic(KeyEvent.VK_h);
	shutdown_tomcat_button.setToolTipText("Click this button to shutdown the Tomcat server");
	
	restart_tomcat_button = new JButton("Restart Tomcat",restartTomcatButtonIcon);
	restart_tomcat_button.addActionListener(rtbl);
	restart_tomcat_button.setEnabled(false);
	//shutdown_button.setMnemonic(KeyEvent.VK_h);
	restart_tomcat_button.setToolTipText("Click this button to restart the Tomcat server(which will shutdown Tomcat server and restart it again");
	
	startup_mysql_button = new JButton("Startup MYSQL", startupMysqlButtonIcon);
	startup_mysql_button.addActionListener(smbl);
	startup_mysql_button.setMnemonic(KeyEvent.VK_S);
	startup_mysql_button.setToolTipText("Click this button to startup the MYSQL server");
	startup_mysql_button.setEnabled(false);

	shutdown_mysql_button = new JButton("Shutdown MYSQL", shutdownMysqlButtonIcon);
	shutdown_mysql_button.addActionListener(smdbl);
	shutdown_mysql_button.setEnabled(false);
	//shutdown_button.setMnemonic(KeyEvent.VK_h);
	shutdown_mysql_button.setToolTipText("Click this button to shutdown the MYSQL server");
	
	restart_mysql_button = new JButton("Restart MYSQL",restartMysqlButtonIcon);
	restart_mysql_button.addActionListener(rmbl);
	restart_mysql_button.setEnabled(false);
	//shutdown_button.setMnemonic(KeyEvent.VK_h);
	restart_tomcat_button.setToolTipText("Click this button to restart the MYSQL server(which will shutdown MYSQL server and restart it again");

	save_button = new JButton("Save Change", saveButtonIcon);
	save_button.addActionListener(sabl);
	//save_button.setMnemonic(KeyEvent.VK_a);
	save_button.setEnabled(false);
	save_button.setToolTipText("Click this button to save the change of build property file");
	
	/*reload_button = new JButton("Reload", reloadButtonIcon);
	reload_button.addActionListener(rbl);
	reload_button.setMnemonic(KeyEvent.VK_R);
	reload_button.setToolTipText("Click this button to reload the build properties file");*/
	//Dictionary.registerBoth(build_button, "CreatePane.Build_Collection", "CreatePane.Build_Collection_Tooltip");

	/*delete_button = new JButton("Delete", deleteButtonIcon);
	 delete_button.addActionListener(dbl);
	delete_button.setEnabled(false);
	delete_button.setMnemonic(KeyEvent.VK_D);
	delete_button.setToolTipText("Click this button to delete the Configuration file");*/
	//Dictionary.registerBoth(cancel_button, "CreatePane.Cancel_Build", "CreatePane.Cancel_Build_Tooltip");*/

	/*exit_button = new JButton("Exit", exitButtonIcon);
	exit_button.setToolTipText("Click this button to exit the viewing");	
	exit_button.addActionListener(ebl);
	exit_button.setEnabled(true);
	exit_button.setMnemonic(KeyEvent.VK_E);*/

	//tomcat buttons
	stbl = null;
	stdbl = null;
	rtbl = null;

	//restart mysql button
	smbl = null;
	smdbl = null;
	rmbl = null;

	//rbl = null;
	//dbl = null;
	//ebl = null;
	
	//read in build properties
	getConfContent();
    }

    /** Any implementation of ActionListener requires this method so that when an 
     **action is performed the appropriate effect can occur.*/

    public void actionPerformed(ActionEvent event) {
    }

    /** This method is callsed to actually layout the components.*/
    public void display() {
	//Create Components.
	//KeyListenerImpl key_listener = new KeyListenerImpl();
	//MouseListenerImpl mouse_listener = new MouseListenerImpl();
	//this.addKeyListener(key_listener);

	// confList_Pane
	confList_pane = new JPanel();
	confList_pane.setBorder(BorderFactory.createLoweredBevelBorder());
	confList_pane.setPreferredSize(LIST_SIZE);
	confList_pane.setSize(LIST_SIZE);
	confList_label = new JLabel("Configuration Files");
	confList_label.setOpaque(true);
	confList_label.setBackground(Configuration.getColor("coloring.workspace_heading_background", false));
	confList_label.setForeground(Configuration.getColor("coloring.workspace_heading_foreground", false));
	//Dictionary.registerText(logList_label, "Log.List");

	// confContent_Pane
	confContent_pane = new JPanel();
	//confContent_pane.setBorder(BorderFactory.createLoweredBevelBorder());
	confContent_pane.setBorder(BorderFactory.createEmptyBorder(10,20,10,20));
	confContent_pane.setBackground(Configuration.getColor("coloring.workspace_selection_background", false));
	confContent_pane.setForeground(Configuration.getColor("coloring.workspace_selection_foreground", false));

	confContent_label = new JLabel("Configuration Content");
	confContent_label.setOpaque(true);
	confContent_label.setBackground(Configuration.getColor("coloring.workspace_heading_background", false));
	confContent_label.setForeground(Configuration.getColor("coloring.workspace_heading_foreground", false));
	//Dictionary.registerText(logContent_label, "Log.Content");
	
	conf_table_model = new ConfTableModel();
	conf_setting_table = new JTable(conf_table_model){
		public String getToolTipText(MouseEvent e){
		    String tip = null;
		    Point p = e.getPoint();
		    int rowIndex = rowAtPoint(p);
		    int colIndex = columnAtPoint(p);
		    int realColumnIndex = convertColumnIndexToModel(colIndex);
		    if (realColumnIndex == 0){
			tip = conf_display[rowIndex][0].toString();
		    } else {
			tip = super.getToolTipText(e);
		    }
		    return tip;
		}
	    };
			
	//Set up tableHeader
	JTableHeader header = conf_setting_table.getTableHeader();
	header.setFont(new Font("Arial", Font.BOLD, 14));
	
	conf_setting_table.setRowHeight(30);
	conf_table_pane = new JScrollPane(conf_setting_table);

	// Layout Components
	confList_pane.setLayout(new BorderLayout());
	JScrollPane conf_treeView = new JScrollPane(conf_tree);
	confList_pane.add(conf_treeView, BorderLayout.CENTER);
	confList_pane.add(confList_label, BorderLayout.NORTH);
	
	confContent_pane.setLayout(new BorderLayout());
	confContent_pane.add(conf_table_pane, BorderLayout.CENTER);
	
	inner_control_pane.setLayout (new BorderLayout());
	inner_control_pane.setBorder(BorderFactory.createEmptyBorder(5,10,5,10));
	inner_control_pane.setPreferredSize(new Dimension(50,50));
	inner_control_pane.setSize(new Dimension(50,50));
	inner_control_pane.add (inner_button_pane, BorderLayout.CENTER);
	
	// Outter Button Layout 
	//outter_button_pane.setLayout (new GridLayout(1,2));
	//outter_button_pane.add(reload_button);
	//outter_button_pane.add(delete_button);
	//outter_button_pane.add(exit_button);
	
	/*outter_control_pane.setLayout (new BorderLayout());
	outter_control_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	outter_control_pane.setPreferredSize(new Dimension(50,50));
	outter_control_pane.setSize(new Dimension(50,50));
	outter_control_pane.add (outter_button_pane, BorderLayout.CENTER);*/

	main_contentPane.setLayout (new BorderLayout());
	main_contentPane.add(confContent_label, BorderLayout.NORTH);
	main_contentPane.add(confContent_pane, BorderLayout.CENTER);
	main_contentPane.add(inner_control_pane, BorderLayout.SOUTH);
	main_conf_pane.add(confList_pane, JSplitPane.LEFT);
	main_conf_pane.add(main_contentPane, JSplitPane.RIGHT);
	main_conf_pane.setDividerLocation(LIST_SIZE.width - 10);	
	
	this.setLayout(new BorderLayout());
	this.add(main_conf_pane, BorderLayout.CENTER);
    }
    
    /*private class ExitButtonListener
	implements ActionListener {
	// Exit the Adminstration tool 
	public void actionPerformed(ActionEvent event) {
	    if (valueChanged) {
		int result = JOptionPane.showConfirmDialog((Component) null, "The value of configuration has been changed,do you really want to exit?", "Exit Confirmation", JOptionPane.YES_NO_OPTION);
		if ( result == JOptionPane.YES_OPTION) {
		    System.exit(1);
		} else if (result == JOptionPane.NO_OPTION) {
		    JOptionPane.showMessageDialog((Component) null,"Press Save Change button to save the change!");
		}
	    } else {
		System.exit(1);
	    }
	}
	}*/

    private class StartupTomcatButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    tomcat_server_up = checkTomcatServer();
	    if (!tomcat_server_up) {
		//ParsingProgress progressBar = new ParsingProgress("Startup Tomcat Server", "Starting up the Tomcat Server", 10);
		ThreadControl threadControl = new ThreadControl();
		Runnable startupTomcatThread = threadControl.new startupTomcatServer("start-tomcat");
		new Thread(startupTomcatThread).start();
		//if over two seconds
		// activae progress bar
		/*progressBar.dispose();
		progressBar.destroy();
		progressBar = null;*/
		threadControl.destroy();
		tomcat_server_up = true;
		JOptionPane.showMessageDialog((Component) null,"The Tomcat server has been Startup successfully!");  
	    } else {
		JOptionPane.showMessageDialog((Component) null,"Tomcat server is running");
	    }
	    changeTomcatButtonPane1();
	}
    }
    
    private class ShutdownTomcatButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    tomcat_server_up = checkTomcatServer();
	    if (tomcat_server_up){
		int result = JOptionPane.showConfirmDialog((Component) null, "Are you sure that you want to shutdown the Tomcat server?", "Shutdown Confirmation", JOptionPane.YES_NO_OPTION);
		if (result == JOptionPane.YES_OPTION) {
		    // ParsingProgress progressBar = new ParsingProgress("Shutdown Tomcat Server", "Shutting down the Tomcat Server", 10);
		    ThreadControl threadControl = new ThreadControl();
		    Runnable shutdownTomcatThread = threadControl.new shutdownTomcatServer("stop-tomcat");
		    new Thread(shutdownTomcatThread).start();
		    /*progressBar.dispose();
		    progressBar.destroy();
		    progressBar = null;*/
		    threadControl.destroy();
		    tomcat_server_up = false;			    
		    JOptionPane.showMessageDialog((Component) null,"The Tomcat server has been Shutdown successfully !");
		} else if (result == JOptionPane.NO_OPTION) {
		    JOptionPane.showMessageDialog((Component) null,"Cancel to Shutdown the Tomcat server!");
		}	
	    } else {
		JOptionPane.showMessageDialog((Component) null,"The Tomcat server was not running!");
	    }
	    changeTomcatButtonPane2();
	}
    }
    
    private class RestartTomcatButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    tomcat_server_up = checkTomcatServer();
	    if (tomcat_server_up) {
		ThreadControl threadControl = new ThreadControl();
		Runnable restartTomcatThread = threadControl.new restartTomcatServer("restart-tomcat");
		new Thread(restartTomcatThread).start();
		threadControl.destroy();
		tomcat_server_up = true;
		JOptionPane.showMessageDialog((Component) null,"The Tomcat server has been Restarted successfully!"); 
	    } else {
		JOptionPane.showMessageDialog((Component) null,"Tomcat server was not running!"); 
	    }
	}
    }


    private class StartupMysqlButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    mysql_server_up = checkMysqlServer();
	    if (!mysql_server_up) {
		ThreadControl threadControl = new ThreadControl();
		Runnable startupMysqlThread = threadControl.new startupMysqlServer("start-mysql");
		new Thread(startupMysqlThread).start();
		threadControl.destroy();
		JOptionPane.showMessageDialog((Component) null,"The MYSQL server has been Startup successfully!");  
		mysql_server_up = true;
	    } else {
		JOptionPane.showMessageDialog((Component) null,"MYSQL server has been running");  	
	    }
	    changeMysqlButtonPane1();
	}
    }  
    
    private class ShutdownMysqlButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    mysql_server_up = checkMysqlServer();
	    if (mysql_server_up){
		int result = JOptionPane.showConfirmDialog((Component) null, "Are you sure that you want to shutdown the MYSQL server?", "Shutdown Confirmation", JOptionPane.YES_NO_OPTION);
		if (result == JOptionPane.YES_OPTION) {
		    ThreadControl threadControl = new ThreadControl();
		    Runnable shutdownMysqlThread = threadControl.new shutdownMysqlServer("stop-mysql");
		    new Thread(shutdownMysqlThread).start();
		    threadControl.destroy();
		    JOptionPane.showMessageDialog((Component) null,"The MYSQL server has been Shutdown successfully !");
		    mysql_server_up = false;
		} else if (result == JOptionPane.NO_OPTION) {
		    JOptionPane.showMessageDialog((Component) null,"Cancel to Shutdown the MYSQL server!");
		}	
	    } else {
		JOptionPane.showMessageDialog((Component) null,"The MYSQL server was not running!");
	    }
	    changeMysqlButtonPane2();
	}
    }

    private class RestartMysqlButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    mysql_server_up = checkMysqlServer();
	    if (mysql_server_up) {
		ThreadControl threadControl = new ThreadControl();
		Runnable restartMysqlThread = threadControl.new restartMysqlServer("restart-mysql");
		new Thread(restartMysqlThread).start();
		threadControl.destroy();
		JOptionPane.showMessageDialog((Component) null,"MYSQL server has been Restarted successfully!");  
		mysql_server_up = true;
	    } else { 
		JOptionPane.showMessageDialog((Component) null,"MYSQL server was not running, use startup button to start the MYSQL server!"); 
	    }
	}
    }

    private class SaveButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    
	    if (!valueChanged) {
		JOptionPane.showMessageDialog((Component) null,"The configuration value has not been changed!"); 
	    } else {
		int result = JOptionPane.showConfirmDialog((Component) null, "Do you really want to save the change", "Save Confirmation", JOptionPane.YES_NO_OPTION);
		if ( result == JOptionPane.YES_OPTION) {
		    saveNewConfSetting();
		    writeConfFile(GAI.build_file);
		    valueChanged = false;
		} else if (result == JOptionPane.NO_OPTION) {
		    valueChanged = true;
		}
	    }
	    
	}
    }
    
    public void saveNewConfSetting(){
	String new_string;
	for (int i=0; i < conf_display.length; i++){
	    if (!conf_display[i][1].toString().matches("^\\s*$")){
		for (int j=0; j < conf_array.size(); j++){
		    if (((String)conf_array.get(j)).startsWith(conf_display[i][1].toString().toLowerCase())){
			new_string = conf_display[i][1].toString()+"="+conf_display[i][2].toString();
			conf_array.set(j, new_string.toLowerCase());
		    }
		}
	    }
	}
	
    }
    

    public void writeConfFile(File file){
	String filename = file.getPath();
	try {
	    BufferedWriter conf_out = new BufferedWriter(new FileWriter(filename));
	    for (int j=0 ; j < conf_array.size(); j++){
		conf_out.write(conf_array.get(j).toString());
		conf_out.newLine();
	    }
	    conf_out.close();
	    getConfContent();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }


    public void getConfContent(){
	System.err.println("GAI Build file:" + GAI.build_file);

	String filename = GAI.build_file.getPath();

	if (!GAI.build_file.exists()){ 
	    JOptionPane.showMessageDialog((Component) null,"Build property file does not exist");  
	    //reload_button.setEnabled(false);
	} else {
	    readFile(filename);
	}
	save_button.setEnabled(true);
    }
    
    public void readFile (String filename) {
	String fileLine;
	/*inside the array will store, the conf_setting[i][0]:Comment, 
	 *conf_setting[i][1]:Para and conf_setting[i][2]:Value
	 * conf_array[] store all the details from the build.properties*/
	conf_array = new ArrayList();
	conf_setting = new Object [rowLength][colLength];
	try {
	    BufferedReader conf_in = new BufferedReader(new FileReader(filename));
	    int i = 0;
	    int j = 0;
	    while ((fileLine = conf_in.readLine()) != null) {
		// This is an empty line
		if (fileLine.matches("^\\s*$")) {
		    // Do Nothing
		} else if (fileLine.matches("##.+")){
		    //This line shows the specific service for these conf setting
		    conf_array.add(fileLine);
		} else 	if (fileLine.matches("#.+")){
		    //This line is Configuration Comment line
		    conf_array.add(fileLine);
		    conf_setting[i][0] = fileLine.substring(1); // use 1 to get rid of the # symbol.
		} else if (!fileLine.matches("#.+") && !fileLine.matches("^\\s*$") ){
		    //This line is Setting line
		    int end_index = fileLine.indexOf("=");
		    conf_setting[i][1] = fileLine.substring(0,end_index).toUpperCase();
		    conf_setting[i][2] = fileLine.substring(end_index+1); 
		    i++; //calculat the number of settings
		    conf_array.add(fileLine);
		} else {
		    // Wrong character in the line
		}
	    }
	    num_of_setting = i;
	    conf_in.close();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
    public void showConfSetting(String conf_param) {
	clearTable();
	int j=0;
	for (int i=0; i< num_of_setting; i++){
	    if (conf_setting[i][1].toString().matches("^("+ conf_param +").+")) {
		if (conf_setting[i][1].toString().matches("TOMCAT.SERVER")){
		    tomcat_server = conf_setting[i][2].toString();
		}
		if (conf_setting[i][1].toString().matches("TOMCAT.PORT")){
		    tomcat_port = conf_setting[i][2].toString();
		}
		if (conf_setting[i][1].toString().matches("MYSQL.ADMIN.USER")){
		    mysql_adminuser = conf_setting[i][2].toString();
		}
		if (conf_setting[i][1].toString().matches("MYSQL.SERVER")){
		    mysql_server = conf_setting[i][2].toString();
		}
		if (conf_setting[i][1].toString().matches("MYSQL.PORT")){
		    mysql_port = conf_setting[i][2].toString();
		}	
		conf_display[j][0] = conf_setting[i][0];
	    	conf_display[j][1] = conf_setting[i][1];
	   	conf_display[j][2] = conf_setting[i][2];
		j++;
	    }
	}
    }
    
    public void clearTable (){
	for (int i=0; i< rowLength; i++){
	    conf_display[i][0] = "";
	    conf_display[i][1] = "";
	    conf_display[i][2] = "";
	}
    }

    private boolean checkTomcatServer (){
	try {
	    if (tomcat_server != "" && tomcat_port != ""){
		String http_string = "http://" + tomcat_server+":"+tomcat_port+"/";
		URL tomcatURL = new URL(http_string);
		HttpURLConnection tomcatConn = (HttpURLConnection) tomcatURL.openConnection();
		tomcatConn.connect();
		//necessary to get a Response, even if you don't care about contents
		InputStream connIn = tomcatConn.getInputStream(); 
		int pageStatus = tomcatConn.getResponseCode();
		if (pageStatus == HttpURLConnection.HTTP_NOT_FOUND) {//404 error
		    //Tomcat is not running
		    tomcat_server_up = false;
		} else {
		    tomcat_server_up = true;
		}  
	    } else {
		System.err.println("Etiher tomcat server or tomcat port was not set properly");
	    }
	} catch (Exception ex) {
	    System.err.println(ex.getMessage());
	}
	return tomcat_server_up;
    }
    
    private boolean checkMysqlServer(){
	Connection conn = null;
	try {
	    String mysql_userName ="gsdl3reader";
	    String mysql_password = ""; 
	    String mysql_url ="jdbc:mysql://"+mysql_server+":"+mysql_port+"/localsite_gs3mgdemo";
	    System.err.println("what is Mysql_URL:" + mysql_url);
	    Class.forName("com.mysql.jdbc.Driver").newInstance();
	    conn = DriverManager.getConnection(mysql_url, mysql_userName, mysql_password);
	    System.err.println("MYSQL Server connection:" + conn);
	    if (conn == null) {
		//System.err.println("Connection is failed");
		mysql_server_up =false;
	    } else {
		mysql_server_up = true;
	    }
	} catch (Exception e) {
	    System.err.println("Cannot connect to database server");
	}
	return mysql_server_up;
    }
	
    class ConfTableModel extends AbstractTableModel {
	String[] columnNames = {"Configuration Parameter",
				"Configuration Value"};
	
	public int getColumnCount(){
	    return columnNames.length;
	}

	public int getRowCount() {
	    return conf_display.length;
	}
	
	public String getColumnName(int col){
	    return columnNames[col];
	}

	public Object getValueAt(int row,int col){
	    return conf_display[row][col+1];
	}
	
	public boolean isCellEditable(int row, int col){
	    if (col == 0){
		return false;
	    } else {
		return true;
	    }
	}

	public void setValueAt(Object value, int row, int col){
	    conf_display[row][col+1] = value;
	    fireTableCellUpdated(row,col+1);
	    valueChanged = true;
	}
    }

    private class ConfTreeListener implements TreeSelectionListener{
	String conf_param;
	public void valueChanged (TreeSelectionEvent e){
	    String option = conf_tree.getLastSelectedPathComponent().toString();
	    if (option == "Compile"){
		if (valueChanged){
		    JOptionPane.showMessageDialog((Component) null,"The configuration setting has been changed, please save the change");
		} else {
		    conf_param = "COMPILE";
		    //conf_tree.setToolTipText("The Configuration Setting for the Compilation ");
		    inner_button_pane.removeAll();
		    inner_button_pane.setLayout(new GridLayout(1,3));
		    inner_button_pane.add(save_button);
		    save_button.setEnabled(true);
		    showConfSetting(conf_param);
		    updateUI();
		}
	    } else if (option == "Tomcat"){
		if (valueChanged) {
		    JOptionPane.showMessageDialog((Component) null,"The configuration setting has been changed, please save the change");
		} else {
		    conf_param ="TOMCAT";
		    showConfSetting(conf_param);
		    //Setup Tomcat button control pane
		    inner_button_pane.removeAll();
		    inner_button_pane.setLayout(new GridLayout(1,3));
		    if (checkTomcatServer()) {
			changeTomcatButtonPane1();
		    } else {
			changeTomcatButtonPane2();
		    }
		    updateUI();
		}
	    } else if (option =="Proxy"){
		if (valueChanged) {
		    JOptionPane.showMessageDialog((Component) null,"The configuration setting has been changed, please save the change");
		} else {	
		    conf_param="PROXY";
		    showConfSetting(conf_param);
		    //conf_tree.setToolTipText("The Configuration Setting for the Proxy Server");
		    inner_button_pane.removeAll();
		    inner_button_pane.setLayout(new GridLayout(1,3));
		    inner_button_pane.add(save_button);
		    save_button.setEnabled(true);
		    updateUI();
		}
	    } else if (option == "Mysql"){
		if (valueChanged) {
		    JOptionPane.showMessageDialog((Component) null,"The configuration setting has been changed, please save the change");
		} else {
		    conf_param="MYSQL";
		    showConfSetting(conf_param);
		    //conf_tree.setToolTipText("The Configuration Setting for the MYSQL Server");
		    // Setup MySQL button control pane
		    inner_button_pane.removeAll();
		    inner_button_pane.setLayout(new GridLayout(1,3));
		    if (checkMysqlServer()) {
			changeMysqlButtonPane1();
		    } else {
			changeMysqlButtonPane2();
		    }
		    updateUI();
		}
	    } else if (option == "GSDL"){
		if (valueChanged) {
		    JOptionPane.showMessageDialog((Component) null,"The configuration setting has been changed, please save the change");
		} else {
		    showConfSetting(conf_param);
		    //conf_tree.setToolTipText("The Configuration Setting for the GSDL");
		    inner_button_pane.removeAll();
		    inner_button_pane.setLayout(new GridLayout(1,3));
		    inner_button_pane.add(save_button);
		    save_button.setEnabled(true);		
		    conf_param ="GSDL";
		    updateUI();
		}
	    } else if (option == "Conf 2"){
		//MysqlInstall mysql = 
		JOptionPane.showMessageDialog((Component) null,"This has not been defined yet!"); 		
	    } else if (option == "Conf 3") {
		clearTable();
		JOptionPane.showMessageDialog((Component) null,"This has not been defined yet!"); 
	    }
	}
    }
    public void modeChanged (int mode){
	return;
    }

    public void gainFocus() {
	return;
    }

    public void changeTomcatButtonPane1 (){
	inner_button_pane.removeAll();
	inner_button_pane.setLayout(new GridLayout(1,3));
	inner_button_pane.add(restart_tomcat_button);
	inner_button_pane.add(shutdown_tomcat_button);
	inner_button_pane.add(save_button);
	restart_tomcat_button.setEnabled (true);
	shutdown_tomcat_button.setEnabled (true);
	save_button.setEnabled (true);	
	updateUI();
    }
    public void changeTomcatButtonPane2 (){
	inner_button_pane.removeAll();
	inner_button_pane.setLayout(new GridLayout(1,3));
	inner_button_pane.add(startup_tomcat_button);
	inner_button_pane.add(shutdown_tomcat_button);
	inner_button_pane.add(save_button);
	shutdown_tomcat_button.setEnabled (true);
	startup_tomcat_button.setEnabled (true);
	save_button.setEnabled (true);
	updateUI();
    }	
    public void changeMysqlButtonPane1 (){
	inner_button_pane.removeAll();
	inner_button_pane.setLayout(new GridLayout(1,3));
	inner_button_pane.add(restart_mysql_button);
	inner_button_pane.add(shutdown_mysql_button);
	inner_button_pane.add(save_button);
	restart_mysql_button.setEnabled (true);
	shutdown_mysql_button.setEnabled (true);
	save_button.setEnabled (true);	
	updateUI();
    }
    public void changeMysqlButtonPane2 (){
	inner_button_pane.removeAll();
	inner_button_pane.setLayout(new GridLayout(1,3));
	inner_button_pane.add(startup_mysql_button);
	inner_button_pane.add(shutdown_mysql_button);
	inner_button_pane.add(save_button);
	shutdown_mysql_button.setEnabled (true);
	startup_mysql_button.setEnabled (true);
	save_button.setEnabled (true);
	updateUI();
    }	
}
       

